/* This file is automatically generated by utils/generate-registry.pl
   Manual changes will be lost. */

int oglm_count_ArrayObjectPNameATI(int param);
int oglm_count_AtomicCounterBufferPName(int param);
int oglm_count_BlendingFactor(int param);
int oglm_count_Buffer(int param);
int oglm_count_BufferPNameARB(int param);
int oglm_count_BufferPointerNameARB(int param);
int oglm_count_BufferStorageTarget(int param);
int oglm_count_BufferTargetARB(int param);
int oglm_count_ClampColorTargetARB(int param);
int oglm_count_ClipPlaneName(int param);
int oglm_count_ColorMaterialParameter(int param);
int oglm_count_ColorTableParameterPName(int param);
int oglm_count_ColorTableTarget(int param);
int oglm_count_ColorTableTargetSGI(int param);
int oglm_count_ContainerType(int param);
int oglm_count_ContextFlagMask(int param);
int oglm_count_ConvolutionParameter(int param);
int oglm_count_ConvolutionTarget(int param);
int oglm_count_ConvolutionTargetEXT(int param);
int oglm_count_CopyBufferSubDataTarget(int param);
int oglm_count_CopyImageSubDataTarget(int param);
int oglm_count_EnableCap(int param);
int oglm_count_FogPName(int param);
int oglm_count_FogParameter(int param);
int oglm_count_FragmentLightParameterSGIX(int param);
int oglm_count_FramebufferAttachmentParameterName(int param);
int oglm_count_FramebufferParameterName(int param);
int oglm_count_FramebufferTarget(int param);
int oglm_count_GetFramebufferParameter(int param);
int oglm_count_GetHistogramParameterPNameEXT(int param);
int oglm_count_GetMinmaxParameterPNameEXT(int param);
int oglm_count_GetMultisamplePNameNV(int param);
int oglm_count_GetPName(int param);
int oglm_count_GetPointervPName(int param);
int oglm_count_GetTextureParameter(int param);
int oglm_count_HintTarget(int param);
int oglm_count_HistogramTarget(int param);
int oglm_count_HistogramTargetEXT(int param);
int oglm_count_IndexMaterialParameterEXT(int param);
int oglm_count_InternalFormatPName(int param);
int oglm_count_InvalidateFramebufferAttachment(int param);
int oglm_count_LightModelParameter(int param);
int oglm_count_LightName(int param);
int oglm_count_LightParameter(int param);
int oglm_count_MapTarget(int param);
int oglm_count_MaterialParameter(int param);
int oglm_count_MinmaxTarget(int param);
int oglm_count_MinmaxTargetEXT(int param);
int oglm_count_ObjectIdentifier(int param);
int oglm_count_PNTrianglesPNameATI(int param);
int oglm_count_PatchParameterName(int param);
int oglm_count_PipelineParameterName(int param);
int oglm_count_PixelCopyType(int param);
int oglm_count_PixelStoreParameter(int param);
int oglm_count_PixelTransferParameter(int param);
int oglm_count_PointParameterNameARB(int param);
int oglm_count_ProgramInterfacePName(int param);
int oglm_count_ProgramPropertyARB(int param);
int oglm_count_ProgramResourceProperty(int param);
int oglm_count_ProgramStagePName(int param);
int oglm_count_ProgramStringProperty(int param);
int oglm_count_ProgramTarget(int param);
int oglm_count_QueryCounterTarget(int param);
int oglm_count_QueryObjectParameterName(int param);
int oglm_count_QueryParameterName(int param);
int oglm_count_QueryTarget(int param);
int oglm_count_RenderbufferParameterName(int param);
int oglm_count_SamplerParameterF(int param);
int oglm_count_SamplerParameterI(int param);
int oglm_count_SeparableTarget(int param);
int oglm_count_SeparableTargetEXT(int param);
int oglm_count_ShaderParameterName(int param);
int oglm_count_ShaderType(int param);
int oglm_count_StringName(int param);
int oglm_count_SubroutineParameterName(int param);
int oglm_count_SyncParameterName(int param);
int oglm_count_TextureEnvMode(int param);
int oglm_count_TextureEnvParameter(int param);
int oglm_count_TextureEnvTarget(int param);
int oglm_count_TextureGenParameter(int param);
int oglm_count_TextureParameterName(int param);
int oglm_count_TextureTarget(int param);
int oglm_count_TransformFeedbackPName(int param);
int oglm_count_UniformBlockPName(int param);
int oglm_count_UniformPName(int param);
int oglm_count_VertexArrayPName(int param);
int oglm_count_VertexAttribEnum(int param);
int oglm_count_VertexAttribPointerPropertyARB(int param);
int oglm_count_VertexAttribPropertyARB(int param);
int oglm_count_VertexShaderTextureUnitParameter(int param);
